const express = require('express');       // 引入Express
const multer  = require('multer');        
const uuid = require('uuid');
const path = require('path');
const fs = require('fs');
const userService = require('./service/user.js'); 
const diaryService = require('./service/diary.js');

const app = express();                    // 初始化Express
const port = 8000;                        // 使用8000端口

const tempDir = 'temp';                   // 临时文件夹
const staticDir = 'public';               // 静态资源文件夹
const uploadDir = 'upload';               // 上传文件正式存储的文件夹: /public/uploads

app.use(express.static(staticDir));
app.use(express.json());

// 获得首页数据
app.post('/getFirstPageData', async function(req, resp) {
  try {
    const data = {
      userInfo: await userService.getUserInfoById('u1'),     // u1用户的信息
      diaries: await diaryService.getAllDiaries(),           // 所有日记
    };
    resp.send(data);  // 向前端返回数据
  } catch (err) {
    // 向前端返回错误信息
    resp.status(500).send(err.message);
  }
});

// 上传文件
const storage = multer.diskStorage({
  destination: tempDir,
  filename: function (req, file, cb) {
    cb(null, uuid.v1() +  file.originalname.substring(file.originalname.lastIndexOf('.')));
  }
});
const upload = multer({ storage });
app.post('/upload', upload.single('file'), function (req, resp) {
  resp.send(req.file.filename);
});

// 发表日记
app.post('/publishDiary', async function (req, resp) {
  try {
    // 获得前端提交的数据(日记数据)
    const params = req.body;
    
    const srcFolder = path.join(__dirname, tempDir);                // 源文件所在的文件夹路径(临时文件夹)
    const desFolder = path.join(__dirname, staticDir, uploadDir);   // 目的文件夹路径(public/uploads)

    // 若上传文件夹不存在则创建之
    if (!fs.existsSync(desFolder)) {
      fs.mkdirSync(desFolder, { recursive: true });
    }
    // 将封面图片和音频文件从临时文件夹移动到正式的上传文件存储位置
    fs.renameSync(path.join(srcFolder, params.coverServerFileName), path.join(desFolder, params.coverServerFileName));
    if (params.audioServerFileName) {
      fs.renameSync(path.join(srcFolder, params.audioServerFileName), path.join(desFolder, params.audioServerFileName));
    }
    
    // 替换参数中的封面图片, 音频文件路径为相对路径
    Object.assign(params, {
      cover: `${uploadDir}/${params.coverServerFileName}`,   // 封面图片的相对路径,
      audioPath: params.audioServerFileName ? `${uploadDir}/${params.audioServerFileName}` : null,    // 音频文件的相对路径
    });

    // 保存日记信息到数据库
    await diaryService.addDiary(params);
    
    // 向前端返回成功标志
    resp.send('OK');
  } catch (err) {
    resp.status(500).send(err.message);
  }
});

// 启动服务器端程序
app.listen(port, function() {
    console.log('启动成功! 快试试吧~ http://localhost:' + port + '/');
});