const mysql = require('mysql');     // 引入mysql

// 创建数据库连接
const pool = mysql.createPool({
    host: 'localhost',              // MySQL所在的主机
    port: 3306,                     // MySQL监听的端口, 默认3306. 若有必要,根据自己的情况修改
    user: 'root',                   // 数据库登录名. 若有必要,根据自己的情况修改
    password: '1234',               // 数据库登录密码. 若有必要,根据自己的情况修改
    database: 'daily_reading',      // 数据库名称
});

// 内部使用的数据库操作函数
// 使用connection执行单条sql命令, params为sql中参数占位符对应参数值(数组)
function _query(connection, sql, params) {
    return new Promise((resolve, reject) => {
        connection.query(sql, params || [], (err, result) => {
            if (err) reject(err);
            resolve(result);
        })
    })
}

/**
 * 执行SQL命令
 * 可用于执行一条需要SQL命令(sql参数为string)
 * 也可用于执行多条SQL命令(sql参数为Array<String>), 此时多条命令将使用一个事务执行.
 * @param {String|Array<String>}  sql 待执行的单条或多条SQL命令
 * @param {Array<any>}  params 与sql命令中参数点位符对应的参数值表.
 * 
 * 例: 
 * query( 'select * from t where id=?', ['a'] );
 * query([ 'update t set age=10 where id=?', 'delete from t where id=?' ], [['a'], ['b']]);
 */
function query(sql, params) {
    return new Promise((resolve, reject) => {
        // 若sql为单条命令, 直接执行
        if (typeof sql === 'string') {
            pool.query(sql, params || [], (err, result) => {
                if (err) reject(err);
                resolve(result);
            });
        }
        // sql为命令数组, 启用事务执行
        else if (Array.isArray(sql) && sql.length && sql.length === params.length) {
            pool.getConnection(function (err, connection) {     // 获得数据库连接对象
                if (err) reject(err);
                connection.beginTransaction(async err => {      // 启动事务
                    // 若启动事件出错, 则释放连接, 返回
                    if (err) {
                        connection.release();
                        reject(err);
                    }

                    const results = [];     // 用于存储每一条SQL命令的执行结果
                    try {
                        // 依次执行sql数组中的各条SQL指令
                        for (let i = 0; i < sql.length; i++) {
                            results.push(await _query(connection, sql[i], params[i]));
                        }
                        // 提交事务
                        connection.commit(err => {
                            connection.release();   // 释放数据库连接
                            if (err) reject(err);
                            resolve(results);       // 返回结果
                        });
                    } catch (e) {
                        // 异常时回滚事务
                        connection.rollback(() => {
                            connection.release();
                        });
                        reject(e);
                    }
                });
            });
        }
        // 调用本函数时传入的参数不合法
        else {
            reject('Illegal Argument');
        }
    });
}

/**
 * 构造insert语句(sql)和参数表(values)
 * @param {String} table 表名
 * @param {Array<String>} fields 要插入的字段列表
 * @param {Object} params 参数对象
 * @param {Function} getValue 获取参数值的回调函数: getValue(字段名, 根据字段名取得的默认值)
 * @return {Object} insert语句及参数表: {sql, values} 
 */
function buildInsert(table, fields, params, getValue) {
    const sql = `INSERT INTO ${table} ( ${fields.join(', ')}) values ( ${new Array(fields.length).fill('?').join(', ')} )`;
    const values = [];
    fields.forEach(field => {
        if (!getValue) return values.push(params[field]);
        values.push(getValue(field, params[field]));
    });
    return { sql, values };
}

module.exports = {
    query,
    buildInsert
}