const api = require('../../utils/api');

// 录音机
const recorderManager = wx.getRecorderManager();
// 音频播放器
const audioPlayer = wx.getBackgroundAudioManager();

// 录音选项
const recordOptions = {
  duration: 600000,       // 最长10分钟
  sampleRate: 44100,      // 采集率
  encodeBitRate: 64000,   // 编码率
  format: 'mp3'           // 录音格式
};

// 录音计时器
let recordTimer = null;

Page({
  data: { 
    showRecordPanel: false,            // 是否显示录音机面板
    isRecording: false,                // 是否正在录音
    recordDuration: 0,                 // 当前的录音时长(录音机中红色的那个0:00)
    isAudioPlaying: false,             // 是否正在播放录音
    isChangingAudioPos: false,         // 用户是否正在拖动音频播放进度滑块    
    audioElapsed: 0,                   // 当前已经播放的音频时长(秒)
    diary: {                           // 日记数据
      title: '',                       // 日记标题
      cover: '',                       // 封面图片
      content: '',                     // 日记内容
      audioPath: '',                   // 音频路径
      audioDuration: 0,                // 音频时长
    }
  },
  state: {
    isCancelRecord: false                // 是否用户取消录音
  },
  onLoad: function () {
    // 开始录音
    recorderManager.onStart(() => { this.setData({ isRecording: true }); });
    // 暂停录音
    recorderManager.onPause(() => { this.setData({ isRecording: false }); });
    // 恢复(继续)录音
    recorderManager.onResume(() => { this.setData({ isRecording: true }); });
    // 录音结束(完成)
    recorderManager.onStop((res) => {
      this.setData({ 
        isRecording: false,
        'diary.audioPath': this.state.isCancelRecord ? null : res.tempFilePath,
        'diary.audioDuration': this.state.isCancelRecord ? 0 : Math.floor(res.duration / 1000)
      });
      // 关闭"录音机"面板 
      this.closeRecordPanel();    
    });

    // 开始播放
    audioPlayer.onPlay(() => { this.setData({isAudioPlaying: true}); });
    // 播放暂停
    audioPlayer.onPause(() => { this.setData({isAudioPlaying: false}); });
    // 播放停止
    audioPlayer.onStop(() => { this.setData({isAudioPlaying: false, audioElapsed: 0}); });
    // 播放结束
    audioPlayer.onEnded(() => { this.setData({isAudioPlaying: false, audioElapsed: 0}); });
    // 播放进度变化
    audioPlayer.onTimeUpdate(() => {
      // 若当前用户没有正在拖动播放滑动, 则播放器的时候随播放进度变化
      if (!this.data.isChangingAudioPos) {
        this.setData({audioElapsed: audioPlayer.currentTime});
      }
    });
  },
  // 输入框内容变化, 记录用户输入值
  // 日记"内容"和"标题"统一处理
  onFieldInput: function(e) {
    this.data.diary[e.currentTarget.dataset.field] = e.detail.value;
  },
  // 设置封面图片
  setCover: function() {
    wx.chooseImage({
      count: 1, // 最多选择1张图片
      sizeType: ['original'], // 上传原图
      sourceType: ['album', 'camera'],  // 图片来源于相册或相机
      success: (res) => {
        const filePaths = res.tempFilePaths;  // 选中的图片文件路径(数组)
        if (filePaths && filePaths.length) {
          this.setData({'diary.cover': filePaths[0]});
        }
      }
    });
  },
  // 打开录音面板
  showRecordPanel: function() {
    this.setData({ showRecordPanel: true, recordDuration: 0 });
    this.state.isCancelRecord = false;
    // 启动录音计时器, 每秒更新一次计时器
    recordTimer = setInterval(()=>{
      if (this.data.isRecording) {    // 若未在录音(暂停录音)则不再累加计时器
        this.setData({recordDuration: this.data.recordDuration + 1});
      }
    }, 1000);
    // 开始录音
    recorderManager.start(recordOptions);
  },
  // 关闭录音机面板
  closeRecordPanel: function() {
    this.setData({ showRecordPanel: false });
    clearInterval(recordTimer);
  },
  // 暂停/恢复录音
  pauseOrResumeRecord: function() {
    if (this.data.isRecording) {
      recorderManager.pause();
    } else {
      recorderManager.resume();
    }
  },
  // 取消录音(用户点击了录音机左侧的叉)
  cancelRecord: function() {
    if (this.data.isRecording) {
      this.state.isCancelRecord = true;
      recorderManager.stop();
    }
  },
  // 完成录音(用户轻触了录音机"完成"按钮)
  finishRecord: function() {
    recorderManager.stop();
  },
  // 删除录音
  removeAudio: function() {
    this.setData({ 
      'diary.audioPath': null, 
      'diary.audioDuration': 0 
    });
  },
  // 音频播放/暂停
  audioPlayOrPause: function (e) {
    // 若当前点击的音频正在播放则暂停并返回
    if (this.data.isAudioPlaying) {
      audioPlayer.pause();
    } else {
      audioPlayer.src = this.data.diary.audioPath;
      audioPlayer.title = '您的录音';
      audioPlayer.play();
    }
  },
  // 用户正在拖动播放进度滑块
  onChangingAudioPos: function(e) {
    this.setData({ 
      isChangingAudioPos: true,        // 标记当前用户正在播放进度滑块
      audioElapsed: e.detail.value     // 当前播放时长与滑块位置同步
    });
  },
  // 进度滑块位置改变回调函数
  onChangeAudioPos: function(e) {
    audioPlayer.seek(e.detail.value);             // 将播放点跳至滑块对应位置
    if (audioPlayer.paused) audioPlayer.play();   // 若音频处于暂停状态, 拖动结束后自动开始播放
    this.setData({ isChangingAudioPos: false});   // 记录当前已结束滑块拖动
  },
  // 发表日记
  publishDiary: async function() {
    const diary = this.data.diary;
    
    // 表单校验
    const fieldCheck = function(field, errMsg) {
      if (!diary[field]) {
        wx.showToast({ title: errMsg, icon: 'none'});
        return false;
      }
      return true;
    }    
    if (!fieldCheck('title', '标题还是要有滴~')) return; 
    if (!fieldCheck('content', '说点什么吧~')) return;
    if (!fieldCheck('cover', '美照来一张~')) return;
    
    try {
      diary.coverServerFileName = await api.uploadFile('/upload', null, diary.cover);          // 上传封面图片
      if (diary.audioPath) {  // 音频可选
        diary.audioServerFileName = await api.uploadFile('/upload', null, diary.audioPath);      // 上传音频
      }
      await api.request('/publishDiary', diary);  // 提交日记数据
      wx.showToast({ title: '日记已发表~'});
      wx.navigateBack();                          // 返回上一页
    } catch(e) {
      wx.showToast({ title: '发表日记失败', icon: 'none'});
    }
  }
})